# -*- coding: utf8 -*-

from collective.dms.mailcontent.browser.reply_form import ReplyForm
from collective.dms.mailcontent.browser.views import OMCustomAddForm
from collective.dms.mailcontent.browser.views import OMEdit
from collective.dms.mailcontent.testing import INTEGRATION
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from z3c.relationfield.relation import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest2 as unittest


class TestSettings(unittest.TestCase):
    """ Base class to test views """

    layer = INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Contributor'])
        self.folder = api.content.create(container=self.portal, type='Folder', id='folder', title='Folder')
        config = [{'name': u'ud', 'token': u'ud'}]
        self.dir = api.content.create(container=self.portal, type='directory', id='dir', title='Directory',
                                      organization_types=config, organization_levels=config,
                                      position_types=config)
        self.c1 = api.content.create(container=self.dir, type='organization', id='imio', title='IMIO')
        self.request = self.folder.REQUEST

    def clean_request(self):
        for key in ('_hide_irn', '_auto_ref'):
            if key in self.request:
                self.request.other.pop(key)

    def om_params(self, view):
        return {
            'IDublinCore.title': u'test', 'internal_reference_no': view.widgets['internal_reference_no'].value,
            # 'sender': self.c1, 'recipients': [self.c1]
        }

    def test_settings(self):
        # check default config
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 10)
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_talexpression'), u"python:'test-out/'+number")
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_edit_irn'), u'show')
        self.assertTrue(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                       'outgoingmail_increment_number'))
        self.assertTrue(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                       'outgoingmail_today_mail_date'))

        # testing OM views: default parameters
        add = OMCustomAddForm(self.folder, self.request)
        add.portal_type = 'dmsoutgoingmail'
        add.update()
        self.assertNotIn('_hide_irn', self.request.keys())
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(add.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.assertEquals(add.widgets['internal_reference_no'].value, u'test-out/10')
        obj = add.createAndAdd(self.om_params(add))
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertFalse(hasattr(om, '_auto_ref'))
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 11)
        self.clean_request()
        edit = OMEdit(om, self.request)
        edit.update()
        self.assertNotIn('_hide_irn', self.request.keys())
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(edit.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.clean_request()

        # set outgoingmail_increment_number to False
        # => no number incrementation because irn field is editable
        api.portal.set_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                       'outgoingmail_increment_number', False)
        add.update()
        self.assertNotIn('_hide_irn', self.request.keys())
        self.assertEquals(self.request['_auto_ref'], False)
        self.assertEquals(add.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.assertEquals(add.widgets['internal_reference_no'].value, u'test-out/11')
        obj = add.createAndAdd(self.om_params(add))
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertEquals(om._auto_ref, False)
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 11)  # No increment
        self.clean_request()
        edit = OMEdit(om, self.request)
        edit.update()
        self.assertNotIn('_hide_irn', self.request.keys())
        self.assertEquals(self.request['_auto_ref'], False)
        self.assertEquals(edit.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.clean_request()

        # set outgoingmail_increment_number to False and outgoingmail_edit_irn to hide
        # => number incrementation because irn field is not editable
        api.portal.set_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                       'outgoingmail_edit_irn', 'hide')
        add.update()
        self.assertEquals(self.request['_hide_irn'], True)
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(add.widgets['internal_reference_no'].mode, 'hidden')
        self.assertEquals(add.widgets['internal_reference_no'].value, u'')
        obj = add.createAndAdd(self.om_params(add))
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertEquals(om.internal_reference_no, u'test-out/11')
        self.assertFalse(hasattr(om, '_auto_ref'))
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 12)
        self.clean_request()
        edit = OMEdit(om, self.request)
        edit.update()
        self.assertEquals(self.request['_hide_irn'], True)
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(edit.widgets['internal_reference_no'].mode, 'hidden')
        self.clean_request()

        # set outgoingmail_increment_number to False and outgoingmail_edit_irn to reply
        # => number incrementation because irn field is not editable
        api.portal.set_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                       'outgoingmail_edit_irn', 'reply')
        add.update()
        self.assertEquals(self.request['_hide_irn'], True)
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(add.widgets['internal_reference_no'].mode, 'hidden')
        self.assertEquals(add.widgets['internal_reference_no'].value, u'')
        obj = add.createAndAdd(self.om_params(add))
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertEquals(om.internal_reference_no, u'test-out/12')
        self.assertFalse(hasattr(om, '_auto_ref'))
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 13)
        self.clean_request()
        edit = OMEdit(om, self.request)
        edit.update()
        # is not a response
        self.assertEquals(self.request['_hide_irn'], True)
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(edit.widgets['internal_reference_no'].mode, 'hidden')
        self.clean_request()
        # is a response but no workflow
        setattr(om, '_is_response', True)
        edit.update()
        self.assertEquals(api.content.get_state(om, default='no_workflow'), 'no_workflow')
        self.assertEquals(edit.is_initial_state(), True)  # has no workflow
        self.assertNotIn('_hide_irn', self.request.keys())
        self.assertEquals(self.request['_auto_ref'], False)
        self.assertEquals(edit.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.clean_request()
        # is a response, workflow and initial state
        pw = self.portal.portal_workflow
        pw.setChainForPortalTypes(['dmsoutgoingmail'], 'intranet_workflow')
        edit.update()
        self.assertEquals(api.content.get_state(om), 'internal')
        self.assertEquals(edit.is_initial_state(), True)
        self.assertNotIn('_hide_irn', self.request.keys())
        self.assertEquals(self.request['_auto_ref'], False)
        self.assertEquals(edit.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.clean_request()
        # is a response, workflow and not initial state
        api.content.transition(om, 'submit')
        edit.update()
        self.assertEquals(api.content.get_state(om), 'pending')
        self.assertEquals(edit.is_initial_state(), False)
        self.assertEquals(self.request['_hide_irn'], True)
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(edit.widgets['internal_reference_no'].mode, 'hidden')
        self.clean_request()

        # Testing reply view
        intids = getUtility(IIntIds)
        im = api.content.create(container=self.folder, type='dmsincomingmail', id='im', title=u'I mail',
                                sender=[RelationValue(intids.getId(self.c1))], external_reference_no=u'xx/1',
                                treating_groups=['Administrators'])
        api.portal.set_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                       'outgoingmail_edit_irn', 'show')
        api.portal.set_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                       'outgoingmail_increment_number', True)
        reply = ReplyForm(im, self.request)
        reply.update()
        self.assertEquals(self.request['_irn'], 'test-in/10')
        self.assertEquals(self.request['_hide_irn'], False)
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(reply.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.assertEquals(reply.widgets['internal_reference_no'].value, u'test-out/13')
        obj = reply.createAndAdd(self.om_params(reply))
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertFalse(hasattr(om, '_auto_ref'))
        self.assertEquals(om._is_response, True)
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 14)
        self.clean_request()

        # set outgoingmail_increment_number to False
        # => no number incrementation because irn field is editable
        api.portal.set_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                       'outgoingmail_increment_number', False)
        reply.update()
        self.assertEquals(self.request['_hide_irn'], False)
        self.assertEquals(self.request['_auto_ref'], False)
        self.assertEquals(reply.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.assertEquals(reply.widgets['internal_reference_no'].value, u'test-out/14')
        obj = reply.createAndAdd(self.om_params(reply))
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertEquals(om._auto_ref, False)
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 14)  # No increment
        self.clean_request()

        # set outgoingmail_increment_number to False and outgoingmail_edit_irn to hide
        # => number incrementation because irn field is not editable
        api.portal.set_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                       'outgoingmail_edit_irn', 'hide')
        reply.update()
        self.assertEquals(self.request['_hide_irn'], True)
        self.assertNotIn('_auto_ref', self.request.keys())
        self.assertEquals(reply.widgets['internal_reference_no'].mode, 'hidden')
        self.assertEquals(reply.widgets['internal_reference_no'].value, u'')
        obj = reply.createAndAdd(self.om_params(reply))
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertEquals(om.internal_reference_no, u'test-out/14')
        self.assertFalse(hasattr(om, '_auto_ref'))
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 15)
        self.clean_request()

        # set outgoingmail_increment_number to False and outgoingmail_edit_irn to reply
        # => number incrementation because irn field is not editable
        api.portal.set_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                       'outgoingmail_edit_irn', 'reply')
        reply.update()
        self.assertEquals(self.request['_hide_irn'], False)
        self.assertEquals(self.request['_auto_ref'], False)
        self.assertEquals(reply.widgets['internal_reference_no'].mode, 'input')  # not hidden
        self.assertEquals(reply.widgets['internal_reference_no'].value, u'test-out/15')
        obj = reply.createAndAdd(self.om_params(reply))
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertEquals(om._auto_ref, False)
        self.assertEquals(api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                         'outgoingmail_number'), 15)  # No increment
        self.clean_request()



# -*- coding: utf8 -*-

from collective.dms.basecontent.dmsdocument import IDmsDocument
from collective.dms.mailcontent import dmsmail
from collective.dms.mailcontent.testing import INTEGRATION
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.utils import createContentInContainer
from plone.registry.interfaces import IRegistry
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.interface import Invalid
from zope.intid import IIntIds

import datetime
import unittest2 as unittest
from zope.lifecycleevent import modified


class TestContentTypes(unittest.TestCase):
    """Base class to test new content types"""

    layer = INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.folder = api.content.create(container=self.portal, type='Folder', id='folder', title='Folder')


class TestDmsmailMethods(TestContentTypes):

    def test_reception_date_default_value(self):
        reception_date = dmsmail.receptionDateDefaultValue('')
        current_date = datetime.datetime.now()
        self.assertEqual(reception_date.date(), current_date.date())
        self.assertEqual(reception_date.hour, current_date.hour)
        self.assertEqual(reception_date.minute, current_date.minute)

    def test_original_date_default_value(self):
        self.assertEqual(dmsmail.originalMailDateDefaultValue(''), datetime.date.today() - datetime.timedelta(3))

    def test_validateIndexValueUniqueness(self):
        imail1 = createContentInContainer(self.portal, 'dmsincomingmail', **{'internal_reference_no': '12345',
                                                                             'title': 'Test 1'})
        # test with container as context, value doesn't exist
        self.assertEquals(dmsmail.validateIndexValueUniqueness(self.portal, dmsmail.IDmsIncomingMail,
                                                               'internal_reference_number', '54321'), None)
        # test with container as context, value exists
        self.assertRaisesRegexp(Invalid, u"This value is already used", dmsmail.validateIndexValueUniqueness,
                                *[self.portal, dmsmail.IDmsIncomingMail, 'internal_reference_number', '12345'])
        # test with object as context, value doesn't exist
        self.assertEquals(dmsmail.validateIndexValueUniqueness(imail1, dmsmail.IDmsIncomingMail,
                                                               'internal_reference_number', '54321'), None)
        # test with object as context, value exists on the same object
        self.assertEquals(dmsmail.validateIndexValueUniqueness(imail1, dmsmail.IDmsIncomingMail,
                                                               'internal_reference_number', '12345'), None)
        # test with object as context and a sub element in the folder, value exists on the same object
        createContentInContainer(imail1, 'dmsmainfile', **{'title': 'File 1'})
        self.assertEquals(dmsmail.validateIndexValueUniqueness(imail1, dmsmail.IDmsIncomingMail,
                                                               'internal_reference_number', '12345'), None)
        # test with object as context, value exists on a different object too
        imail2 = createContentInContainer(self.portal, 'dmsincomingmail',
                                          **{'internal_reference_no': '12345', 'title': 'Test 2'})
        self.assertRaisesRegexp(Invalid, u"This value is already used", dmsmail.validateIndexValueUniqueness,
                                *[imail2, dmsmail.IDmsIncomingMail, 'internal_reference_number', '12345'])
        # test with empty value
        imail3 = createContentInContainer(self.portal, 'dmsincomingmail',
                                          **{'internal_reference_no': '', 'title': 'Test 2'})
        self.assertEquals(dmsmail.validateIndexValueUniqueness(self.portal, dmsmail.IDmsIncomingMail,
                                                               'internal_reference_number', ''), None)
        self.assertEquals(dmsmail.validateIndexValueUniqueness(imail3, dmsmail.IDmsIncomingMail,
                                                               'internal_reference_number', ''), None)

    def test_evaluateInternalReference(self):
        self.assertEquals(dmsmail.evaluateInternalReference(self.portal, self.portal.REQUEST,
                          'collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_number',
                          'collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_talexpression'),
                          'test-in/10')

    def test_incrementIncomingMailNumber(self):
        registry = getUtility(IRegistry)
        old_value = registry['collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_number']
        imail1 = createContentInContainer(self.portal, 'dmsincomingmail', **{'internal_reference_no': '12345',
                                                                             'title': 'Test 1'})
        self.assertEquals(registry['collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_number'],
                          old_value+1)
        self.assertEquals(imail1.internal_reference_no, '12345')
        # we create a dmsincomingmail without internal_reference_no, it will be generated
        imail2 = createContentInContainer(self.portal, 'dmsincomingmail', **{'title': 'Test 2'})
        self.assertEquals(imail2.internal_reference_no, 'test-in/11')

    def test_title(self):
        imail1 = createContentInContainer(self.portal, 'dmsincomingmail', **{'internal_reference_no': '12345',
                                                                             'title': 'Test 1'})
        self.assertEquals(imail1.Title(), "12345 - Test 1")

    def test_indexes(self):
        imail1 = createContentInContainer(self.portal, 'dmsincomingmail', **{'internal_reference_no': '12345',
                                                                             'title': 'Test 1'})
        # get brain
        brains = self.portal.portal_catalog(path='/'.join(imail1.getPhysicalPath()))
        self.assertEquals(len(brains), 1)
        self.assertEquals(imail1, brains[0].getObject())
        # search by title
        brains = self.portal.portal_catalog(Title="12345")
        self.assertEquals(len(brains), 1)
        self.assertEquals(imail1, brains[0].getObject())
        # search by SearchableText: must be done in doc tests or robot
        brains = self.portal.portal_catalog(SearchableText="12345")

    def test_get_replied(self):
        intids = getUtility(IIntIds)
        imail1 = createContentInContainer(self.folder, 'dmsincomingmail', **{'title': 'In 1'})
        imail2 = createContentInContainer(self.folder, 'dmsincomingmail', **{'title': 'In 2'})
        omail1 = createContentInContainer(self.folder, 'dmsoutgoingmail', **{'title': 'Out 1'})
        omail2 = createContentInContainer(self.folder, 'dmsoutgoingmail', **{'title': 'Out 2'})
        self.assertEqual(omail1.get_replied(), None)
        omail1.reply_to = [RelationValue(intids.getId(imail1))]
        modified(omail1)
        self.assertEqual(omail1.get_replied(), imail1)
        omail1.reply_to = [RelationValue(intids.getId(imail2)), RelationValue(intids.getId(imail1))]
        modified(omail1)
        self.assertEqual(omail1.get_replied(), imail2)
        self.assertListEqual(omail1.get_replied(first=False), [imail2, imail1])
        imail2.reply_to = [RelationValue(intids.getId(omail2))]
        modified(imail2)
        self.assertEqual(omail2.get_replied(), imail2)
        omail2.reply_to = [RelationValue(intids.getId(omail1))]
        modified(omail2)
        self.assertEqual(omail2.get_replied(), imail2)
        self.assertEqual(omail2.get_replied(intf=IDmsDocument), omail1)



from z3c.form.browser.radio import RadioFieldWidget
from zope.interface import alsoProvides
from zope.schema import Choice
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from plone.autoform.interfaces import IFormFieldProvider
from plone.autoform import directives as form
from plone.supermodel import model

from collective.dms.mailcontent import _


sending_types = SimpleVocabulary([
    SimpleTerm(value='normal', title=_(u'Normal')),
    SimpleTerm(value='registered', title=_(u'Registered')),
])


class ISendingType(model.Schema):
    """Sending type behavior"""
    sending_type = Choice(title=_(u"Sending type"),
                          vocabulary=sending_types,
                          default='normal',
                          required=False)
    form.widget(sending_type=RadioFieldWidget)


alsoProvides(ISendingType, IFormFieldProvider)



from imio.helpers.content import get_relations
from . import _
from collective import dexteritytextindexer
from collective.contact.widget.schema import ContactChoice
from collective.contact.widget.schema import ContactList
from collective.dms.basecontent.dmsdocument import DmsDocument
from collective.dms.basecontent.dmsdocument import IDmsDocument
from collective.dms.basecontent.relateddocs import RelatedDocs
from imio.helpers.emailer import validate_email_address
from imio.helpers.emailer import validate_email_addresses
from plone import api
from plone.app.textfield import RichText
from plone.autoform import directives as form
from plone.dexterity.schema import DexteritySchemaPolicy
from plone.directives.form.value import default_value
from plone.formwidget.datetime.z3cform.widget import DateFieldWidget
from plone.formwidget.datetime.z3cform.widget import DatetimeFieldWidget
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from plone.supermodel.directives import fieldset
from Products.CMFPlone.utils import getToolByName
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form import validator
from zope import schema
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError
from zope.interface import implements
from zope.interface import Invalid

import datetime


def validateIndexValueUniqueness(context, type_interface, index_name, value):
    """
        check at 'portal_type' 'context' creation if 'index' 'value' is uniqueness
    """
    # if the value is empty, we don't check anything
    if not value:
        return
    catalog = getToolByName(context, 'portal_catalog')
    brains = catalog.searchResults(**{index_name: value})
    if not type_interface.providedBy(context):
        # we create the dmsincomingmail, the context is the container
        if brains:
            raise Invalid(_(u"This value is already used"))
    else:
        # we edit the type, the context is itself
        # if multiple brains (normally not possible), we are sure there are other objects with same index value
        if len(brains) > 1 or (len(brains) == 1 and brains[0].UID != context.UID()):
            raise Invalid(_(u"This value is already used"))


class InternalReferenceBaseValidator(validator.SimpleFieldValidator):

    type_interface = None

    def validate(self, value):
        # we call the already defined validators
        # super(InternalReferenceValidator, self).validate(value)
        try:
            validateIndexValueUniqueness(self.context, self.type_interface,
                                         'internal_reference_number', value)
        except Invalid:
            raise Invalid(_(u"This value is already used. A good value would be: ${good_value}",
                            mapping={'good_value': internalReferenceIncomingMailDefaultValue(self)}))


class IDmsIncomingMail(IDmsDocument):
    """ """

    original_mail_date = schema.Date(
        title=_(u'Original Mail Date'),
        required=False,)
    form.widget(original_mail_date=DateFieldWidget)

    reception_date = schema.Datetime(title=_(u'Reception Date'), required=False)
    form.widget('reception_date', DatetimeFieldWidget, show_time=True)

    external_reference_no = schema.TextLine(
        title=_(u"External Reference Number"),
        required=False,)

    dexteritytextindexer.searchable('internal_reference_no')
    internal_reference_no = schema.TextLine(
        title=_(u"Internal Reference Number"),
        required=False,)

    sender = ContactList(
        title=_(u'Sender'),
        required=True)

    recipients = ContactList(
        title=_(u'Recipients'),
        required=False)

    reply_to = RelatedDocs(
        title=_(u"In Reply To"),
        required=False,
        object_provides=('collective.dms.mailcontent.dmsmail.IDmsIncomingMail',
                         'collective.dms.mailcontent.dmsmail.IDmsOutgoingMail'),
        display_backrefs=True)

    form.order_before(sender='treating_groups')
    form.order_after(recipients='sender')
    form.order_before(original_mail_date='treating_groups')
    form.order_before(reception_date='treating_groups')
    form.order_before(internal_reference_no='treating_groups')
    form.order_before(external_reference_no='treating_groups')
    form.order_before(reply_to='treating_groups')

    form.order_after(related_docs='recipient_groups')
    form.order_after(notes='related_docs')


class InternalReferenceIncomingMailValidator(InternalReferenceBaseValidator):

    type_interface = IDmsIncomingMail


validator.WidgetValidatorDiscriminators(InternalReferenceIncomingMailValidator,
                                        field=IDmsIncomingMail['internal_reference_no'])


@default_value(field=IDmsIncomingMail['reception_date'])
def receptionDateDefaultValue(data):
    # return the current datetime
    return datetime.datetime.now()


@default_value(field=IDmsIncomingMail['original_mail_date'])
def originalMailDateDefaultValue(data):
    # return 3 days before
    return datetime.date.today() - datetime.timedelta(3)


def evaluateInternalReference(context, request, number_registry_name, talexpression_registry_name):
    # return a generated internal reference number
    registry = getUtility(IRegistry)
    # we get the following mail number, stored in registry
    number = registry.get(number_registry_name) or 1
    # we get the portal
    try:
        portal_state = getMultiAdapter((context, request), name=u'plone_portal_state')
        settings_view = getMultiAdapter((portal_state.portal(), request), name=u'dmsmailcontent-settings')
    except ComponentLookupError:
        return 'Error getting view...'
    # we evaluate the expression
    expression = registry.get(talexpression_registry_name)
    value = settings_view.evaluateTalExpression(expression, context, request, portal_state.portal(), number)
    return value


@default_value(field=IDmsIncomingMail['internal_reference_no'])
def internalReferenceIncomingMailDefaultValue(data):
    """
        Default value of internal_reference_no for dmsincomingmail
    """
    return evaluateInternalReference(data.context, data.request,
                                     'collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_number',
                                     'collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                     'incomingmail_talexpression').decode('utf8')


class DmsIncomingMail(DmsDocument):
    """ """
    implements(IDmsIncomingMail)
    __ac_local_roles_block__ = False

    def Title(self):
        if self.internal_reference_no is None:
            return self.title.encode('utf8')
        return "%s - %s" % (self.internal_reference_no.encode('utf8'), self.title.encode('utf8'))


def incrementIncomingMailNumber(incomingmail, event):
    """ Increment the value in registry """
    # if internal_reference_no is empty, we force the value.
    # useless if the internal_reference_no field is hidden (in this case,
    #                                                       default value must be empty to bypass validator)
    # useless to manage automatically the internal_reference_no value without user action
    if not incomingmail.internal_reference_no:
        internal_reference_no = evaluateInternalReference(incomingmail, incomingmail.REQUEST,
                                                          'collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                          'incomingmail_number',
                                                          'collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                          'incomingmail_talexpression')
        incomingmail.internal_reference_no = internal_reference_no
        incomingmail.reindexObject(idxs=('Title', 'internal_reference_number', 'SearchableText', 'sortable_title'))
    registry = getUtility(IRegistry)
    registry['collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_number'] += 1


class IDmsOutgoingMail(IDmsDocument):
    """ """

    mail_date = schema.Date(title=_(u'Mail Date'), required=False)

    dexteritytextindexer.searchable('internal_reference_no')
    internal_reference_no = schema.TextLine(
        title=_(u"Internal Reference Number"),
        required=False, )

    sender = ContactChoice(
        title=_(u'Sender'),
        required=False)

    recipients = ContactList(
        title=_(u'Recipients'),
        required=True)

    reply_to = RelatedDocs(
        title=_(u"In Reply To"),
        required=False,
        object_provides=('collective.dms.mailcontent.dmsmail.IDmsIncomingMail',
                         'collective.dms.mailcontent.dmsmail.IDmsOutgoingMail'),
        display_backrefs=True)

    external_reference_no = schema.TextLine(
        title=_(u"External Reference Number"),
        required=False,)

    form.order_before(sender='treating_groups')
    form.order_before(recipients='treating_groups')
    form.order_before(mail_date='treating_groups')
    form.order_before(internal_reference_no='treating_groups')
    form.order_before(external_reference_no='treating_groups')
    form.order_before(reply_to='treating_groups')

    form.order_after(related_docs='recipient_groups')
    form.order_after(notes='related_docs')


@default_value(field=IDmsOutgoingMail['mail_date'])
def mailDateDefaultValue(data):
    # return the day date
    today = api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                           'outgoingmail_today_mail_date')
    if today:
        return datetime.date.today()


@default_value(field=IDmsOutgoingMail['internal_reference_no'])
def internalReferenceOutgoingMailDefaultValue(data):
    """
        Default value of internal_reference_no for dmsoutgoingmail
    """
    return evaluateInternalReference(data.context, data.request,
                                     'collective.dms.mailcontent.browser.settings.IDmsMailConfig.outgoingmail_number',
                                     'collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                     'outgoingmail_talexpression').decode('utf8')


class InternalReferenceOutgoingMailValidator(InternalReferenceBaseValidator):
    type_interface = IDmsOutgoingMail


validator.WidgetValidatorDiscriminators(InternalReferenceOutgoingMailValidator,
                                        field=IDmsOutgoingMail['internal_reference_no'])


def incrementOutgoingMailNumber(outgoingmail, event):
    """ Increment the value in registry """
    # if internal_reference_no is empty, we force the value.
    # useful if the internal_reference_no field is hidden (in this case,
    #                                                      default value must be empty to bypass validator)
    # useful to manage automatically the internal_reference_no value without user action
    if not outgoingmail.internal_reference_no:
        internal_reference_no = evaluateInternalReference(outgoingmail, outgoingmail.REQUEST,
                                                          'collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                          'outgoingmail_number',
                                                          'collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                          'outgoingmail_talexpression')
        outgoingmail.internal_reference_no = internal_reference_no
        outgoingmail.reindexObject(idxs=('Title', 'internal_reference_number', 'SearchableText', 'sortable_title'))
    if getattr(outgoingmail, '_auto_ref', True):
        registry = getUtility(IRegistry)
        registry['collective.dms.mailcontent.browser.settings.IDmsMailConfig.outgoingmail_number'] += 1


class DmsOutgoingMail(DmsDocument):
    """ """
    implements(IDmsOutgoingMail)

    def Title(self):
        if self.internal_reference_no is None:
            return self.title.encode('utf8')
        return "%s - %s" % (self.internal_reference_no.encode('utf8'), self.title.encode('utf8'))

    def get_replied(self, first=True, intf=IDmsIncomingMail):
        normals = [obj for obj in get_relations(self, 'reply_to', backrefs=False, as_obj=True)
                   if intf.providedBy(obj)]
        backs = [obj for obj in get_relations(self, 'reply_to', backrefs=True, as_obj=True)
                 if intf.providedBy(obj)]
        for obj in backs:
            if obj not in normals:
                normals.append(obj)
        if first:
            return normals and normals[0] or None
        else:
            return normals


class IOutgoingEmail(model.Schema):
    """ """

    email_status = schema.TextLine(
        title=_(u"Email status"),
        required=False,
    )
    form.write_permission(email_status='cmf.ManagePortal')

    email_subject = schema.TextLine(
        title=_(u"Email subject"),
    )

    email_sender = schema.TextLine(
        title=_(u"Email sender"),
        constraint=validate_email_address,
    )

    email_recipient = schema.TextLine(
        title=_(u"Email recipient"),
        description=_(u"Multiple values must be separated by a comma."),
        constraint=validate_email_addresses,
    )

    email_cc = schema.TextLine(
        title=_(u"Email cc"),
        # description=_(u"Multiple values must be separated by a comma."),
        required=False,
        constraint=validate_email_addresses,
    )

    email_attachments = schema.List(
        title=_(u"Email attachments"),
        required=False,
        value_type=schema.Choice(vocabulary=u'collective.dms.mailcontent.email_attachments_voc'),
    )
    form.widget('email_attachments', CheckBoxFieldWidget, multiple='multiple', size=10)

    email_body = RichText(
        title=_(u"Email body"),
        allowed_mime_types=(u"text/html",),
    )


class IFieldsetOutgoingEmail(IOutgoingEmail):
    """ """

    fieldset(
        'email',
        label=_(u"Email"),
        fields=['email_status', 'email_subject', 'email_sender', 'email_recipient', 'email_cc', 'email_attachments',
                'email_body']
    )


class DmsIncomingMailSchemaPolicy(DexteritySchemaPolicy):
    """ """

    def bases(self, schemaName, tree):
        return (IDmsIncomingMail, )


class DmsOutgoingMailSchemaPolicy(DexteritySchemaPolicy):
    """ """

    def bases(self, schemaName, tree):
        return (IDmsOutgoingMail, )



# -*- coding: utf-8 -*-
#
# File: setuphandlers.py
#
# GNU General Public License (GPL)
#

import logging
logger = logging.getLogger('collective.dms.mailcontent: setuphandlers')
from zope.component import queryUtility, getUtility
from plone.registry.interfaces import IRegistry
from zope.i18n.interfaces import ITranslationDomain

def isNotGoodProfile(context):
    return context.readDataFile("collective_dms_mailcontent_marker.txt") is None


def postInstall(context):
    """Called as at the end of the setup process. """
    # the right place for your custom code

    if isNotGoodProfile(context): return
    registry = getUtility(IRegistry)
    if not registry.get('collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_number'):
        registry['collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_number'] = 1
    if not registry.get('collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_talexpression'):
        registry['collective.dms.mailcontent.browser.settings.IDmsMailConfig.incomingmail_talexpression'] = u"python:'in/'+number"
    if not registry.get('collective.dms.mailcontent.browser.settings.IDmsMailConfig.outgoingmail_number'):
        registry['collective.dms.mailcontent.browser.settings.IDmsMailConfig.outgoingmail_number'] = 1
    if not registry.get('collective.dms.mailcontent.browser.settings.IDmsMailConfig.outgoingmail_talexpression'):
        registry['collective.dms.mailcontent.browser.settings.IDmsMailConfig.outgoingmail_talexpression'] = u"python:'out/'+number"



# -*- coding: utf8 -*-

from plone.app.testing import PloneWithPackageLayer
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting
import collective.dms.mailcontent

COLLECTIVE_DMS_MAILCONTENT = PloneWithPackageLayer(
    zcml_package=collective.dms.mailcontent,
    zcml_filename='testing.zcml',
    additional_z2_products=(),
    gs_profile_id='collective.dms.mailcontent:testing',
    name="COLLECTIVE_DMS_MAILCONTENT")

INTEGRATION = IntegrationTesting(
    bases=(COLLECTIVE_DMS_MAILCONTENT, ),
    name="INTEGRATION")

FUNCTIONAL = FunctionalTesting(
    bases=(COLLECTIVE_DMS_MAILCONTENT, ),
    name="FUNCTIONAL")



from collective.dms.basecontent.dmsdocument import IDmsDocument
from collective.dms.mailcontent.dmsmail import IDmsIncomingMail
from collective.dms.mailcontent.dmsmail import IDmsOutgoingMail
from plone.indexer import indexer
from Products.CMFPlone.utils import base_hasattr
from Products.PluginIndexes.common.UnIndex import _marker


@indexer(IDmsIncomingMail)
def im_internal_reference_number_indexer(obj):
    """Indexer of 'internal_reference_number' for IDmsIncomingMail. Stores internal_reference_no.
    Specific indexer method to avoid acquisition of dmsincomingmail contained elements.
    """
    if obj.internal_reference_no:
        return obj.internal_reference_no
    return _marker


@indexer(IDmsOutgoingMail)
def om_internal_reference_number_indexer(obj):
    """Indexer of 'internal_reference_number' for IDmsOutgoingMail. Stores internal_reference_no.
    Specific indexer method to avoid acquisition of dmsoutgoingmail contained elements.
    """
    if obj.internal_reference_no:
        return obj.internal_reference_no
    return _marker


@indexer(IDmsIncomingMail)
def im_external_reference_number_indexer(obj):
    """Indexer of 'external_reference_number' for IDmsIncomingMail. Stores external_reference_no.
    Specific indexer method to avoid acquisition of dmsincomingmail contained elements.
    """
    if obj.external_reference_no:
        return obj.external_reference_no
    return _marker


@indexer(IDmsOutgoingMail)
def om_external_reference_number_indexer(obj):
    """Indexer of 'external_reference_number' for IDmsOutgoingMail. Stores external_reference_no.
    Specific indexer method to avoid acquisition of dmsoutgoingmail contained elements.
    """
    if obj.external_reference_no:
        return obj.external_reference_no
    return _marker


def add_parent_organizations(obj, index):
    for org in obj.get_organizations_chain():
        index.append('l:%s' % org.UID())


def relations_index(obj, attr):
    if not base_hasattr(obj, attr):
        return _marker
    value = getattr(obj, attr)
    if not value:
        return _marker
    index = []
    if not isinstance(value, list):
        value = [value]
    for rel in value:
        if rel.isBroken():
            continue
        related = rel.to_object
        index.append(related.UID())

        if related.portal_type == 'organization':
            add_parent_organizations(related, index)
        elif related.portal_type == 'held_position':
            add_parent_organizations(related.get_organization(), index)
    if index:
        # make unique items
        return list(set(index))
    return _marker


@indexer(IDmsDocument)
def sender_index(obj):
    """Indexer of 'sender_index' for IDmsDocument

    Stores:
        * the sender UIDs
        * the organizations chain UIDs if the sender is an organization or a held position, prefixed by 'l:'
    """
    return relations_index(obj, 'sender')


@indexer(IDmsDocument)
def recipients_index(obj):
    """Indexer of 'recipients_index' for IDmsDocument

    Stores:
        * the recipient UIDs
        * the organizations chain UIDs if the recipient is an organization or a held position, prefixed by 'l:'
    """
    return relations_index(obj, 'recipients')



from plone import api
from zope.component import getUtility
from zope.i18n import ITranslationDomain
from zope.i18nmessageid import MessageFactory

_ = MessageFactory("collective.dms.mailcontent")


def initialize(context):
    """Initializer called when used as a Zope 2 product."""


def _tr(msgid, domain='collective.dms.mailcontent', mapping=None):
    translation_domain = getUtility(ITranslationDomain, domain)
    sp = api.portal.get().portal_properties.site_properties
    return translation_domain.translate(msgid, target_language=sp.getProperty('default_language', 'fr'),
                                        mapping=mapping)



from collective.dms.mailcontent import _
from DateTime import DateTime
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.supermodel import model
from plone.z3cform import layout
from Products.CMFCore.Expression import Expression
from Products.PageTemplates.Expressions import getEngine
from zope import schema
from zope.publisher.browser import BrowserPage
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


im_irn_conditions = SimpleVocabulary([SimpleTerm(u"show", title=_("Always show")),
                                      SimpleTerm(u"hide", title=_("Always hide"))])

om_irn_conditions = SimpleVocabulary([SimpleTerm(u"show", title=_("Always show")),
                                      SimpleTerm(u"hide", title=_("Always hide")),
                                      SimpleTerm(u"reply", title=_("Show only when replying"))])


class IDmsMailConfig(model.Schema):
    """
    Configuration of dms mail
    """

    model.fieldset(
        'incomingmail',
        label=_(u"Incoming mail"),
        fields=['incomingmail_number', 'incomingmail_talexpression']
    )

    incomingmail_number = schema.Int(
        title=_(u'Number of next incoming mail'),
        description=_(u"This value is used as 'number' variable in linked tal expression"))

    incomingmail_talexpression = schema.TextLine(
        title=_(u"Incoming mail internal reference default value expression"),
        description=_(u"Tal expression where you can use portal, number, context, request, date as variable")
    )

    model.fieldset(
        'outgoingmail',
        label=_(u"Outgoing mail"),
        fields=['outgoingmail_number', 'outgoingmail_talexpression', 'outgoingmail_edit_irn',
                'outgoingmail_increment_number', 'outgoingmail_today_mail_date']
    )

    outgoingmail_number = schema.Int(
        title=_(u'Number of next outgoing mail'),
        description=_(u"This value is used as 'number' variable in linked tal expression"))

    outgoingmail_talexpression = schema.TextLine(
        title=_(u"Outgoing mail internal reference default value expression"),
        description=_(u"Tal expression where you can use portal, number, context, request, date as variable")
    )

    outgoingmail_edit_irn = schema.Choice(
        title=_(u"Internal reference number edition field"),
        vocabulary=om_irn_conditions,
        default='show'
    )

    outgoingmail_increment_number = schema.Bool(
        title=_(u"Increment number used in internal reference number"),
        description=_(u"If False, number will not be incremented only if user can edit internal reference number"),
        default=True
    )

    outgoingmail_today_mail_date = schema.Bool(
        title=_(u'Mail date is today'),
        description=_(u"Check if the outgoing mail 'mail date' field will default to today at creation."),
        default=True
    )


class SettingsEditForm(RegistryEditForm):
    """
    Define form logic
    """
    schema = IDmsMailConfig
    label = _(u"Dms Mail settings")


class SettingsView(BrowserPage):
    """
    View which wrap the settings form using ControlPanelFormWrapper to a HTML boilerplate frame.
    """

    def __call__(self):
        view_factor = layout.wrap_form(SettingsEditForm, ControlPanelFormWrapper)
        view = view_factor(self.context, self.request)
        return view()

    def evaluateTalExpression(self, expression, context, request, portal, number, **kwargs):
        """
            evaluate the expression, considering portal and number in context
        """
        #evaluate the numerotationTALExpression and pass it obj, lastValue and self
        data = {
            'tool': self,
            'number': str(number),
            'context': context,
            'request': request,
            'portal': portal,
            'date': DateTime(),
        }
        data.update(kwargs)
        res = ''
        try:
            ctx = getEngine().getContext(data)
            res = Expression(expression)(ctx)
        except Exception, msg:
            return 'Error in expression: %s' % msg
        return res



# -*- coding: utf-8 -*-
"""Vocabularies."""
from collective.dms.basecontent.dmsfile import IDmsAppendixFile
from collective.dms.basecontent.dmsfile import IDmsFile
from collective.dms.mailcontent import _tr
from collective.dms.mailcontent.dmsmail import IDmsOutgoingMail
from imio.helpers.content import find
from plone import api
from Products.CMFPlone.utils import safe_unicode
from zope.annotation import IAnnotations
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class EmailAttachmentsVocabulary(object):
    """ Vocabulary listing outgoing mail files and related ones, as possible email attachments """
    implements(IVocabularyFactory)

    def __call__(self, context):
        terms = []
        # first we find context files, unrestrictedly
        brains = find(context=context, unrestricted=True, object_provides=[IDmsAppendixFile.__identifier__,
                      IDmsFile.__identifier__], sort_on='created', sort_order='descending')
        for brain in brains:
            obj = brain._unrestrictedGetObject()
            annot = IAnnotations(obj)
            # Do not include work mailing document (from collective.documentgenerator)
            if 'documentgenerator' in annot and 'need_mailing' in annot['documentgenerator'] and \
                    annot['documentgenerator']['need_mailing']:
                continue
            ftitle = safe_unicode(brain.Title)
            # Do not include filename if the file is generated by application
            if ftitle.lower() == obj.file.filename.lower() or not IDmsAppendixFile.providedBy(obj):
                title = ftitle
            else:
                title = u'{}  (« {} »)'.format(ftitle, obj.file.filename)
            # TODO Mark signed version
            if getattr(brain, 'signed', False):  # from collective.dms.scanbehavior
                typ = u'{} signed'.format(brain.portal_type)
            else:
                typ = brain.portal_type
            terms.append(SimpleTerm(brain.UID, brain.UID,
                                    _tr(u'({}) => ${{title}}'.format(typ), mapping={'title': title})))
        # then we find files of related mails
        pc = api.portal.get_tool('portal_catalog')
        for rv in getattr(context, 'reply_to', []) or []:
            is_om = IDmsOutgoingMail.providedBy(rv.to_object)
            brains = pc.unrestrictedSearchResults(path=rv.to_path, object_provides=[IDmsAppendixFile.__identifier__,
                                                  IDmsFile.__identifier__], sort_on='created', sort_order='descending')
            for brain in brains:
                obj = brain._unrestrictedGetObject()
                ftitle = safe_unicode(brain.Title)
                annot = IAnnotations(obj)
                # Do not include work mailing document when linked object is an outgoingmail
                if 'documentgenerator' in annot and 'need_mailing' in annot['documentgenerator'] and \
                        annot['documentgenerator']['need_mailing']:
                    continue
                ftitle = safe_unicode(brain.Title)
                # Do not include filename if linked object is an outgoingmmail and the file is generated by application
                if ftitle.lower() == obj.file.filename.lower() or (is_om and not IDmsAppendixFile.providedBy(obj)):
                    title = ftitle
                else:
                    title = u'{}  (« {} »)'.format(ftitle, obj.file.filename)
                if getattr(brain, 'signed', False):  # from collective.dms.scanbehavior
                    typ = u'{} signed'.format(brain.portal_type)
                else:
                    typ = brain.portal_type
                terms.append(SimpleTerm(brain.UID, brain.UID,
                                        _tr(u'${{ref}} ({}) => ${{title}}'.format(typ),
                                            mapping={'ref': rv.to_object.internal_reference_no, 'title': title})))

        terms.sort(key=lambda trm: trm.title.lower())
        return SimpleVocabulary(terms)



# encoding: utf-8

from collective.dms.mailcontent.dmsmail import IDmsIncomingMail
from Products.Five import BrowserView


class UtilsMethods(BrowserView):
    """ View containing utils methods """

    def outgoingmail_folder(self):
        """ Get a folder for outgoing mail """
        if IDmsIncomingMail.providedBy(self.context):
            return self.context.__parent__
        return self.context



# -*- coding: utf8 -*-

from collective.dms.mailcontent.testing import INTEGRATION
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from z3c.relationfield.relation import RelationValue
from zope.component import getUtility
from zope.i18n import translate
from zope.intid.interfaces import IIntIds

import unittest2 as unittest


class TestReplyForm(unittest.TestCase):

    layer = INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Contributor'])
        self.folder = api.content.create(container=self.portal, type='Folder', id='folder', title='Folder')
        config = [{'name': u'ud', 'token': u'ud'}]
        self.dir = api.content.create(container=self.portal, type='directory', id='dir', title='Directory',
                                      organization_types=config, organization_levels=config,
                                      position_types=config)
        self.c1 = api.content.create(container=self.dir, type='organization', id='imio', title='IMIO')
        self.request = self.folder.REQUEST
        intids = getUtility(IIntIds)
        self.im = api.content.create(container=self.folder, type='dmsincomingmail', id='im', title=u'I mail',
                                     sender=[RelationValue(intids.getId(self.c1))], external_reference_no=u'xx/1',
                                     treating_groups=['Administrators'])
        self.view = self.im.unrestrictedTraverse('@@reply')

    def test_rf_label(self):
        self.assertEqual(translate(self.view.label), u'Reply to test-in/10 - I mail')

    def ttest_update_fields_irn(self):
        """ done in test_settings """

    def test_rf_updateFields(self):
        self.view.updateFields()
        self.assertEqual(self.view.request['form.widgets.reply_to'], ('/plone/folder/im',))
        self.assertEqual(self.view.request['form.widgets.recipients'], ('/plone/dir/imio',))

    def test_rf_updateWidgets(self):
        self.view.update()
        self.assertEqual(self.view.widgets['IDublinCore.title'].value, u'I mail')
        self.assertEqual(self.view.widgets['treating_groups'].value, ['Administrators'])
        self.assertEqual(self.view.widgets['reply_to'].value, ('/plone/folder/im',))
        self.assertEqual(self.view.widgets['recipients'].value, ('/plone/dir/imio',))
        self.assertEqual(self.view.widgets['external_reference_no'].value, u'xx/1')
        self.assertEqual(self.view.widgets['recipient_groups'].value, ())
        self.assertEqual(self.view.widgets['internal_reference_no'].value, u'test-out/10')

    def test_rf_add(self):
        setRoles(self.portal, TEST_USER_ID, ['Member', 'Manager'])
        self.view.update()
        params = {
            'IDublinCore.title': self.view.widgets['IDublinCore.title'].value,
            'treating_groups': ['Administrators'],
            'internal_reference_no': self.view.widgets['internal_reference_no'].value,
            'reply_to': [self.im],
            'recipient_groups': []
        }
        self.assertEqual(len(self.folder.objectIds()), 1)
        obj = self.view.createAndAdd(params)
        self.assertEqual(len(self.folder.objectIds()), 2)
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertListEqual(om.recipient_groups, [])
        # treating group is different from im
        params['treating_groups'] = ['Reviewers']
        obj = self.view.createAndAdd(params)
        self.assertEqual(len(self.folder.objectIds()), 3)
        om = api.content.find(self.folder, id=obj.id)[0].getObject()
        self.assertListEqual(om.recipient_groups, ['Administrators'])



# -*- coding: utf8 -*-
import unittest2 as unittest

from zope.component import getUtility

from plone.behavior.interfaces import IBehavior
from plone.autoform.interfaces import IFormFieldProvider
from plone.app.testing.helpers import setRoles
from plone.app.testing.interfaces import TEST_USER_NAME, TEST_USER_ID

from ecreall.helpers.testing.base import BaseTest

from collective.dms.mailcontent.testing import INTEGRATION
from collective.dms.mailcontent.behaviors import ISendingType


class TestBehaviors(unittest.TestCase, BaseTest):
    """Tests behaviors"""

    layer = INTEGRATION

    def setUp(self):
        super(TestBehaviors, self).setUp()
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.login(TEST_USER_NAME)
        self.portal.invokeFactory('testtype', 'testitem')
        self.testitem = self.portal['testitem']

    def test_behaviors_installation(self):
        sending_type_behavior = getUtility(IBehavior,
                                           name='collective.dms.mailcontent.behaviors.ISendingType')
        self.assertEqual(sending_type_behavior.interface, ISendingType)
        IFormFieldProvider.providedBy(sending_type_behavior.interface)

    def test_sending_type_fields(self):
        item = self.testitem
        self.assertIsNone(item.getAttributes())
        self.assertTrue(hasattr(item, 'sending_type'))
        self.assertEqual(item.sending_type, 'normal')
        item.sending_type = 'registered'
        self.assertEqual(item.sending_type, 'registered')



# -*- coding: utf8 -*-

import unittest2 as unittest
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
from z3c.relationfield.relation import RelationValue

from plone import api
from plone.app.testing import setRoles, TEST_USER_ID
from plone.dexterity.utils import createContentInContainer

from collective.dms.mailcontent.testing import INTEGRATION
from ..indexers import sender_index, recipients_index


class TestIndexers(unittest.TestCase):
    """Base class to test new content types"""

    layer = INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.intids = getUtility(IIntIds)
        setup = api.portal.get_tool('portal_setup')
        setup.runImportStepFromProfile('profile-collective.contact.core:test_data',
                                       'collective.contact.core-createTestData')
        self.mydirectory = self.portal['mydirectory']
        self.armeedeterre = self.mydirectory['armeedeterre']
        self.corpsa = self.armeedeterre['corpsa']
        self.divisionalpha = self.corpsa['divisionalpha']
        self.degaulle = self.mydirectory['degaulle']
        self.general = self.degaulle['gadt']
        self.omail1 = createContentInContainer(self.portal, 'dmsoutgoingmail',
                                               **{'title': 'Test 1',
                                                  'sender': RelationValue(self.intids.getId(self.general)),
                                                  'recipients': [RelationValue(self.intids.getId(self.general))]})
        self.imail1 = createContentInContainer(self.portal, 'dmsincomingmail',
                                               **{'title': 'Test 1',
                                                  'sender': [RelationValue(self.intids.getId(self.general)),
                                                             RelationValue(self.intids.getId(self.divisionalpha))],
                                                  'recipients': [RelationValue(self.intids.getId(self.general))]})

    def test_sender_index(self):
        # test outgoingmail
        indexer = sender_index(self.omail1)
        # test with held_position
        index_value = indexer()
        self.assertEqual(len(index_value), 2)
        self.assertIn(self.general.UID(), index_value)
        self.assertIn('l:%s' % self.armeedeterre.UID(), index_value)
        # test with organization
        self.omail1.sender = RelationValue(self.intids.getId(self.divisionalpha))
        index_value = indexer()
        self.assertEqual(len(index_value), 4)
        self.assertIn(self.divisionalpha.UID(), index_value)
        self.assertIn('l:%s' % self.divisionalpha.UID(), index_value)
        self.assertIn('l:%s' % self.corpsa.UID(), index_value)
        self.assertIn('l:%s' % self.armeedeterre.UID(), index_value)
        # test incomingmail
        indexer = sender_index(self.imail1)
        # test with held_position
        index_value = indexer()
        self.assertEqual(len(index_value), 5)
        self.assertIn(self.general.UID(), index_value)
        self.assertIn('l:%s' % self.armeedeterre.UID(), index_value)
        # test with organization
        self.assertIn(self.divisionalpha.UID(), index_value)
        self.assertIn('l:%s' % self.divisionalpha.UID(), index_value)
        self.assertIn('l:%s' % self.corpsa.UID(), index_value)
        self.assertIn('l:%s' % self.armeedeterre.UID(), index_value)

    def test_recipients_index(self):
        indexer = recipients_index(self.omail1)
        # test with held_position
        index_value = indexer()
        self.assertEqual(len(index_value), 2)
        self.assertIn(self.general.UID(), index_value)
        self.assertIn('l:%s' % self.armeedeterre.UID(), index_value)
        # test with organization
        self.omail1.recipients.append(RelationValue(self.intids.getId(self.divisionalpha)))
        index_value = indexer()
        self.assertEqual(len(index_value), 5)
        self.assertIn(self.general.UID(), index_value)
        self.assertIn(self.divisionalpha.UID(), index_value)
        self.assertIn('l:%s' % self.divisionalpha.UID(), index_value)
        self.assertIn('l:%s' % self.corpsa.UID(), index_value)
        self.assertIn('l:%s' % self.armeedeterre.UID(), index_value)



# -*- coding: utf8 -*-

from collective.dms.mailcontent import dmsmail
from collective.dms.mailcontent.testing import INTEGRATION
from collective.dms.mailcontent.vocabularies import EmailAttachmentsVocabulary
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.utils import createContentInContainer
from plone.namedfile import NamedBlobFile
from plone.registry.interfaces import IRegistry
from z3c.relationfield import RelationValue
from zope.annotation import IAnnotations
from zope.component import getUtility

import datetime
import unittest2 as unittest
from zope.intid import IIntIds


class TestVocabularies(unittest.TestCase):
    """Test vocabularies"""

    layer = INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.intids = getUtility(IIntIds)
        self.im1 = api.content.create(container=self.portal, type='dmsincomingmail', id='im1', title=u'I mail',
                                      treating_groups=['Administrators'])
        createContentInContainer(self.im1, 'dmsmainfile', title=u'D001', file=NamedBlobFile(filename=u'scanned.pdf'))
        createContentInContainer(self.im1, 'dmsappendixfile', title=u'A001',
                                 file=NamedBlobFile(filename=u'appendix.odt'))
        self.om1 = api.content.create(container=self.portal, type='dmsoutgoingmail', id='om1', title=u'O mail 1',
                                      treating_groups=['Administrators'])
        self.od1 = createContentInContainer(self.om1, 'dmsmainfile', title=u'D011',
                                            file=NamedBlobFile(filename=u'generated.odt'))
        createContentInContainer(self.om1, 'dmsappendixfile', title=u'A011',
                                 file=NamedBlobFile(filename=u'appendix1.pdf'))
        self.om2 = api.content.create(container=self.portal, type='dmsoutgoingmail', id='om2', title=u'O mail 2',
                                      treating_groups=['Administrators'])
        self.od2 = createContentInContainer(self.om2, 'dmsmainfile', title=u'D021',
                                            file=NamedBlobFile(filename=u'generated.odt'))
        createContentInContainer(self.om2, 'dmsappendixfile', title=u'A021',
                                 file=NamedBlobFile(filename=u'appendix2.pdf'))

    def test_EmailAttachmentsVocabulary(self):
        voc = EmailAttachmentsVocabulary()
        self.assertListEqual([t.title for t in voc(self.om1)],
                             [u'(dmsappendixfile) => A011  (\xab appendix1.pdf \xbb)',
                              u'(dmsmainfile) => D011'])
        self.od3 = createContentInContainer(self.om1, 'dmsmainfile', title=u'D012',
                                            file=NamedBlobFile(filename=u'mailing.odt'))
        annot = IAnnotations(self.od3)
        annot['documentgenerator'] = {'need_mailing': False}
        self.assertListEqual([t.title for t in voc(self.om1)],
                             [u'(dmsappendixfile) => A011  (\xab appendix1.pdf \xbb)',
                              u'(dmsmainfile) => D011',
                              u'(dmsmainfile) => D012'])
        annot['documentgenerator'] = {'need_mailing': True}
        # mailing work document is avoided
        self.assertListEqual([t.title for t in voc(self.om1)],
                             [u'(dmsappendixfile) => A011  (\xab appendix1.pdf \xbb)',
                              u'(dmsmainfile) => D011'])
        # link to im1
        self.om1.reply_to = [RelationValue(self.intids.getId(self.im1))]
        self.assertListEqual([t.title for t in voc(self.om1)],
                             [u'(dmsappendixfile) => A011  (\xab appendix1.pdf \xbb)',
                              u'(dmsmainfile) => D011',
                              u'test-in/10 (dmsappendixfile) => A001  (\xab appendix.odt \xbb)',
                              u'test-in/10 (dmsmainfile) => D001  (\xab scanned.pdf \xbb)'])
        # link to im1 and om2
        self.om1.reply_to = [RelationValue(self.intids.getId(self.im1)), RelationValue(self.intids.getId(self.om2))]
        self.assertListEqual([t.title for t in voc(self.om1)],
                             [u'(dmsappendixfile) => A011  (\xab appendix1.pdf \xbb)',
                              u'(dmsmainfile) => D011',
                              u'test-in/10 (dmsappendixfile) => A001  (\xab appendix.odt \xbb)',
                              u'test-in/10 (dmsmainfile) => D001  (\xab scanned.pdf \xbb)',
                              u'test-out/11 (dmsappendixfile) => A021  (\xab appendix2.pdf \xbb)',
                              u'test-out/11 (dmsmainfile) => D021'])
        self.od4 = createContentInContainer(self.om2, 'dmsmainfile', title=u'D022',
                                            file=NamedBlobFile(filename=u'mailing.odt'))
        annot = IAnnotations(self.od4)
        annot['documentgenerator'] = {'need_mailing': False}
        self.assertListEqual([t.title for t in voc(self.om1)],
                             [u'(dmsappendixfile) => A011  (\xab appendix1.pdf \xbb)',
                              u'(dmsmainfile) => D011',
                              u'test-in/10 (dmsappendixfile) => A001  (\xab appendix.odt \xbb)',
                              u'test-in/10 (dmsmainfile) => D001  (\xab scanned.pdf \xbb)',
                              u'test-out/11 (dmsappendixfile) => A021  (\xab appendix2.pdf \xbb)',
                              u'test-out/11 (dmsmainfile) => D021',
                              u'test-out/11 (dmsmainfile) => D022'])
        annot['documentgenerator'] = {'need_mailing': True}
        self.assertListEqual([t.title for t in voc(self.om1)],
                             [u'(dmsappendixfile) => A011  (\xab appendix1.pdf \xbb)',
                              u'(dmsmainfile) => D011',
                              u'test-in/10 (dmsappendixfile) => A001  (\xab appendix.odt \xbb)',
                              u'test-in/10 (dmsmainfile) => D001  (\xab scanned.pdf \xbb)',
                              u'test-out/11 (dmsappendixfile) => A021  (\xab appendix2.pdf \xbb)',
                              u'test-out/11 (dmsmainfile) => D021'])



# -*- coding: utf-8 -*-

from collective.dms.basecontent.browser.views import DmsDocumentEdit
from collective.dms.mailcontent.dmsmail import IDmsOutgoingMail
from plone import api
from plone.dexterity.browser.add import DefaultAddForm
from plone.dexterity.browser.add import DefaultAddView


def DmsOutgoingMailUpdateWidgets(the_form):  # noqa
    """
        Widgets update method for add, edit and reply !
    """
    # context can be the folder in add or an im in reply.
    # store current_user used in sub product imio.dms.mail
    the_form.current_user = api.user.get_current()
    if (not the_form.current_user.has_role(['Manager', 'Site Administrator']) and
            the_form.request.get('_hide_irn', False)):
        the_form.widgets['internal_reference_no'].mode = 'hidden'
        # we empty value to bypass validator when creating object. Reference will be managed automatically
        if not IDmsOutgoingMail.providedBy(the_form.context):
            the_form.widgets['internal_reference_no'].value = ''


class OMEdit(DmsDocumentEdit):
    """
        Edit form redefinition to customize fields.
    """

    def is_initial_state(self):
        pw = api.portal.get_tool('portal_workflow')
        wfs = pw.getWorkflowsFor(self.context)
        if not wfs:
            return True  # when no workflow, consider it's the initial state
        if api.content.get_state(self.context) == wfs[0].initial_state:
            return True
        return False

    def updateFields(self):
        super(OMEdit, self).updateFields()
        edit_irn = api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                  'outgoingmail_edit_irn')
        if edit_irn == 'hide' or (edit_irn == 'reply' and (not getattr(self.context, '_is_response', False) or
                                  not self.is_initial_state())):
            self.request['_hide_irn'] = True
        elif not api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                'outgoingmail_increment_number'):
            self.request['_auto_ref'] = False  # add it to request than can be used in default method, etc...

    def updateWidgets(self):
        super(OMEdit, self).updateWidgets()
        DmsOutgoingMailUpdateWidgets(self)


class OMCustomAddForm(DefaultAddForm):

    portal_type = 'dmsoutgoingmail'

    def updateFields(self):
        super(OMCustomAddForm, self).updateFields()
        edit_irn = api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                  'outgoingmail_edit_irn')
        if edit_irn in ('hide', 'reply'):
            self.request['_hide_irn'] = True
        elif not api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                'outgoingmail_increment_number'):
            self.request['_auto_ref'] = False

    def updateWidgets(self):
        super(OMCustomAddForm, self).updateWidgets()
        DmsOutgoingMailUpdateWidgets(self)
        # the following doesn't work
        # self.widgets['ITask.assigned_user'].value = [api.user.get_current().getId()]

    def add(self, object):
        if not self.request.get('_auto_ref', True):
            setattr(object, '_auto_ref', False)
        super(OMCustomAddForm, self).add(object)


class AddOM(DefaultAddView):

    form = OMCustomAddForm



# -*- coding: utf-8 -*-
from datetime import date, datetime, time
import logging

from plone import api

from Products.contentmigration.field import migrateField

logger = logging.getLogger('collective.dms.mailcontent: upgrade. ')


def v2(context):
    default_time = time(10, 0)
    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog.searchResults({'portal_type': 'dmsincomingmail'})
    for brain in brains:
        obj = brain.getObject()
        reception_date = obj.reception_date
        if isinstance(reception_date, date):
            obj.reception_date = datetime.combine(reception_date, default_time)


def v3(context):
    catalog = api.portal.get_tool('portal_catalog')
    setup = api.portal.get_tool('portal_setup')
    setup.runAllImportStepsFromProfile('profile-collective.dms.mailcontent:default')
    brains = catalog.searchResults({'portal_type': 'dmsincomingmail'})
    for brain in brains:
        obj = brain.getObject()
        obj.reindexObject(idxs=('Title', 'SearchableText', 'sortable_title'))


def v4(context):
    setup = api.portal.get_tool('portal_setup')
    setup.runImportStepFromProfile('profile-collective.dms.mailcontent:default', 'typeinfo')
    catalog = api.portal.get_tool('portal_catalog')
    migrated = False
    for brain in catalog.searchResults(portal_type=['dmsincomingmail', 'dmsoutgoingmail']):
        obj = brain.getObject()
        if migrateField(obj, {'fieldName': 'in_reply_to', 'newFieldName': 'reply_to'}):
            migrated = True
    logger.info("%s object fields were migrated" % (migrated and 'Some' or 'None'))


def v5(context):
    setup = api.portal.get_tool('portal_setup')
    setup.runImportStepFromProfile('profile-collective.dms.mailcontent:default', 'catalog')
    catalog = api.portal.get_tool('portal_catalog')
    nb = 0
    for brain in catalog.searchResults(portal_type=['dmsincomingmail', 'dmsoutgoingmail']):
        nb += 1
        obj = brain.getObject()
        obj.reindexObject(idxs=['sender'])
    logger.info("%d objects were migrated" % nb)


def v6(context):
    setup = api.portal.get_tool('portal_setup')
    setup.runImportStepFromProfile('profile-collective.dms.mailcontent:default', 'catalog')
    setup.runImportStepFromProfile('profile-collective.dms.mailcontent:default', 'plone.app.registry')
    catalog = api.portal.get_tool('portal_catalog')
    nb = 0
    for brain in catalog.searchResults(portal_type=['dmsincomingmail', 'dmsoutgoingmail']):
        nb += 1
        obj = brain.getObject()
        obj.reindexObject(idxs=['recipients'])
    logger.info("%d objects were migrated" % nb)


def v7(context):
    catalog = api.portal.get_tool('portal_catalog')
    nb = 0
    for brain in catalog.searchResults(portal_type=['dmsincomingmail']):
        nb += 1
        obj = brain.getObject()
        sender = obj.sender
        if sender and not isinstance(sender, list):
            obj.sender = [sender]
            obj.reindexObject(idxs=['sender_index'])
    logger.info("%d objects were migrated" % nb)


def v8(context):
    """ Avoid warning about unresolved dependencies """
    setup = api.portal.get_tool('portal_setup')
    step = 'dmsmailcontent-postInstall'
    registry = setup.getImportStepRegistry()
    registry._registered.get(step)['dependencies'] = (u'catalog', u'controlpanel', u'plone.app.registry', u'rolemap',
                                                      u'typeinfo')
    setup._p_changed = True
    logger.info("Import step dependency corrected")


def v10(context):
    """ Upgrade indexes """
    setup = api.portal.get_tool('portal_setup')
    setup.runImportStepFromProfile('profile-collective.dms.mailcontent:default', 'catalog')
    catalog = api.portal.get_tool('portal_catalog')
    catalog.manage_reindexIndex(ids=['sender_index', 'recipients_index'])
    logger.info("Catalog updated")


def v11(context):
    setup = api.portal.get_tool('portal_setup')
    setup.runImportStepFromProfile('profile-collective.dms.mailcontent:default', 'catalog')
    catalog = api.portal.get_tool('portal_catalog')
    nb = 0
    for brain in catalog.searchResults(portal_type=['dmsincomingmail', 'dmsoutgoingmail']):
        nb += 1
        obj = brain.getObject()
        obj.reindexObject(idxs=['external_reference_number'])
    logger.info("%d objects were migrated" % nb)



# -*- coding: utf-8 -*-
"""Reply form."""
from collective.dms.mailcontent import _
from collective.dms.mailcontent.browser.views import DmsOutgoingMailUpdateWidgets
from plone import api
from plone.dexterity.browser.add import DefaultAddForm
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.utils import addContentToContainer
from Products.CMFPlone.utils import safe_unicode
from zc.relation.interfaces import ICatalog
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError
from zope.intid.interfaces import IIntIds


class ReplyForm(DefaultAddForm):

    """Form to reply to an incoming mail."""

    description = u""
    portal_type = "dmsoutgoingmail"
    reply_backrefs = True

    def __init__(self, context, request, ti=None):
        super(ReplyForm, self).__init__(context, request, ti=ti)
        self.linked_paths = None

    def _get_linked_mails(self, imail):
        if self.linked_paths is not None:
            return self.linked_paths
        ret = ['/'.join(imail.getPhysicalPath())]
        # get directly imail linked mails
        if imail.reply_to:
            ret.extend([rv.to_path for rv in imail.reply_to])
        # we get backrefs too (depending of an option ?)
        if self.reply_backrefs:
            intids = getUtility(IIntIds)
            catalog = getUtility(ICatalog)
            imail_id = intids.getId(imail)
            for ref in catalog.findRelations({'to_id': imail_id, 'from_attribute': 'reply_to'}):
                if ref.from_path not in ret:
                    ret.append(ref.from_path)
        self.linked_paths = tuple(ret)
        return self.linked_paths

    @property
    def label(self):
        return _(u"Reply to ${ref}", mapping={'ref': safe_unicode(self.context.Title())})

    def update_fields_irn(self):
        """ update fields regarding irn setting """
        edit_irn = api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                  'outgoingmail_edit_irn')
        self.request['_hide_irn'] = True
        # a user can edit irn. we don't use number incrementation
        if edit_irn in ('show', 'reply'):
            self.request['_hide_irn'] = False
            if not api.portal.get_registry_record('collective.dms.mailcontent.browser.settings.IDmsMailConfig.'
                                                  'outgoingmail_increment_number'):
                self.request['_auto_ref'] = False

    def updateFields(self):
        super(ReplyForm, self).updateFields()
        imail = self.context
        # put original mail irn in request to be used in irn expression
        self.request['_irn'] = imail.internal_reference_no
        self.update_fields_irn()
        if self.request.get('masterID'):  # in MS anonymous call
            return
        form = self.request.form
        # Completing form values wasn't working anymore, but relations must be set here too !
        # We need to put a value only if key doesn't exist, otherwise the user modifications in form aren't kept
        # Because updateFields is also called after submission
        if "form.widgets.reply_to" not in form:
            form["form.widgets.reply_to"] = self._get_linked_mails(imail)
        if "form.widgets.recipients" not in form:
            form["form.widgets.recipients"] = tuple([sd.to_path for sd in imail.sender])

    def updateWidgets(self, prefix=None):
        super(ReplyForm, self).updateWidgets(prefix=prefix)
        imail = self.context
        self.widgets["IDublinCore.title"].value = safe_unicode(imail.title)
        self.widgets["treating_groups"].value = imail.treating_groups
        if self.request.get('masterID'):  # in MS anonymous call
            return
        self.widgets["reply_to"].value = self._get_linked_mails(imail)
        self.widgets["recipients"].value = tuple([sd.to_path for sd in imail.sender])
        if imail.external_reference_no:
            self.widgets["external_reference_no"].value = imail.external_reference_no
        if imail.recipient_groups:
            self.widgets["recipient_groups"].value = imail.recipient_groups
        DmsOutgoingMailUpdateWidgets(self)

    def add_content(self, obj):
        """Is overrided in inherited view"""
        try:
            utils_view = getMultiAdapter((self.context, self.request), name=u'cdmc-utils')
        except ComponentLookupError:
            return 'Error getting cdmc-utils view...'
        container = utils_view.outgoingmail_folder()
        return container, addContentToContainer(container, obj)

    def add(self, obj):
        """Create outgoing mail in outgoing-mail folder."""
        setattr(obj, '_is_response', True)

        if not self.request.get('_auto_ref', True):
            setattr(obj, '_auto_ref', False)
        # python:request.get('_auto_ref', True) and 'S%04d' % int(number) or 'S/%s/1' % request.get('_irn', '')

        container, new_object = self.add_content(obj)

        # we made sure to add incomingmail treating_groups in recipient_groups if a recipient group has replied
        otgs = obj.treating_groups
        if not isinstance(otgs, (list, tuple)):  # can be a unique value like in imio.dms.mail
            otgs = [otgs]
        intids = getUtility(IIntIds)
        omgs = list(obj.recipient_groups or [])
        for rel in obj.reply_to or []:
            im = intids.getObject(rel.to_id)
            itgs = im.treating_groups
            if not isinstance(itgs, (list, tuple)):  # can be a unique value like in imio.dms.mail
                itgs = [itgs]
            missings = set(itgs) - (set(otgs) | set(omgs))
            for missing in missings:
                obj.recipient_groups.append(missing)
        if omgs != obj.recipient_groups:
            obj.reindexObject(idxs=['recipient_groups'])

        fti = getUtility(IDexterityFTI, name=self.portal_type)
        if fti.immediate_view:
            self.immediate_view = "/".join(
                [container.absolute_url(), new_object.id, fti.immediate_view]
            )
        else:
            self.immediate_view = "/".join(
                [container.absolute_url(), new_object.id]
            )
